"""pgserviceparser GUI - A graphical interface for managing PostgreSQL connection service files."""

from .compat import QtWidgets  # noqa: F401
from .service_widget import PGServiceParserWidget  # noqa: F401
